#[allow(unused_imports)]
pub(crate) use super::components::auth::casbin::casbin_event_consumer::{
    handle_event_for_watcher, launch_casbin_rule_events_consumer,
};
#[allow(unused_imports)]
pub(crate) use super::components::auth::casbin::*;
pub(crate) use super::components::base::{SchedulerMetaInfo, SharedSchedulerMetaInfo};
pub(crate) use super::components::health_checker::loop_health_check;
pub(crate) use super::components::helper::handle_response;
pub(crate) use super::components::operation_log_consumer::{
    loop_operate_logs, send_option_operation_log_pair,
};
pub(crate) use super::db;
pub(crate) use super::db::common::helper::*;
pub(crate) use super::db::common::{model as common_model, state, types};
pub(crate) use super::db::extension::*;
pub(crate) use super::db::model;

pub(crate) use common_model::PaginateData;

pub(crate) use delicate_utils::consensus_message::security::{self, SecurityLevel};
pub(crate) use delicate_utils::consensus_message::service_binding;
pub(crate) use delicate_utils::consensus_message::{
    executor_processor as delicate_utils_executor_processor,
    health_check as delicate_utils_health_check, task as delicate_utils_task,
    task_log as delicate_utils_task_log,
};
pub(crate) use delicate_utils::error::{AuthServiceError, CommonError};
pub(crate) use delicate_utils::helper_utils::get_unique_id_string;
pub(crate) use delicate_utils::prelude::*;
pub(crate) use delicate_utils::uniform_data::*;

pub(crate) use std::cell::RefCell;
pub(crate) use std::collections::{HashMap, HashSet};
pub(crate) use std::convert::{AsRef, TryFrom};
pub(crate) use std::env;
pub(crate) use std::fmt::Debug;
pub(crate) use std::iter::Iterator;
pub(crate) use std::ops::Deref;
pub(crate) use std::rc::Rc;
pub(crate) use std::str::FromStr;
pub(crate) use std::string::ToString;
pub(crate) use std::task::{Context, Poll};
pub(crate) use std::time::Duration;
pub(crate) use std::time::SystemTime;
pub(crate) use std::vec::IntoIter;

pub(crate) use futures::future::{join, join3, ok, JoinAll, Ready};

pub(crate) use cached::proc_macro::cached;
pub(crate) use cached::TimedSizedCache;
pub(crate) use casbin::prelude::*;
pub(crate) use casbin::{
    Enforcer, EventData as CasbinEventData, RbacApi, Watcher as CasbinWatcher,
};

pub(crate) use chrono::{DateTime, Duration as ChronoDuration, Local, NaiveDateTime, Timelike};

pub(crate) use diesel::mysql::Mysql;
pub(crate) use diesel::prelude::*;
pub(crate) use diesel::query_builder::{AsQuery, AstPass, Query, QueryFragment};
pub(crate) use diesel::query_dsl::methods::LoadQuery;
pub(crate) use diesel::r2d2::CustomizeConnection;
pub(crate) use diesel::result::Error as DieselError;
pub(crate) use diesel::sql_types;

pub(crate) use actix_cors::Cors;
pub(crate) use actix_session::{CookieSession, Session, UserSession};
pub(crate) use actix_web::client::Client as RequestClient;
pub(crate) use actix_web::dev::Decompress;
pub(crate) use actix_web::dev::Payload;
pub(crate) use actix_web::dev::{
    HttpResponseBuilder, Service, ServiceRequest, ServiceResponse, Transform,
};
pub(crate) use actix_web::http::StatusCode;
pub(crate) use actix_web::middleware::Logger as MiddlewareLogger;
pub(crate) use actix_web::rt::spawn as rt_spawn;
pub(crate) use actix_web::rt::time::{
    delay_for as rt_delay_for, interval, timeout as rt_timeout, Timeout as RtTimeout,
};
pub(crate) use actix_web::web::{self, Data as ShareData};
pub(crate) use actix_web::{get, post, App, HttpRequest, HttpResponse, HttpServer};
pub(crate) use actix_web::{Error as ActixWebError, Result};
pub(crate) use awc::{JsonBody, SendClientRequest};

pub(crate) use anyhow::Result as AnyResut;
pub(crate) use async_channel::{Receiver as AsyncReceiver, Sender as AsyncSender};
pub(crate) use async_lock::RwLock;
pub(crate) use delay_timer::prelude::*;
pub(crate) use diesel::query_dsl::RunQueryDsl;
pub(crate) use dotenv::dotenv;
pub(crate) use tracing::{debug, error, info, info_span, span, Instrument, Level};
pub(crate) use tracing_subscriber::FmtSubscriber;

pub(crate) use flexi_logger::{
    writers::FileLogWriter, Age, Cleanup, Criterion, FileSpec, Naming, WriteMode,
};
pub(crate) use regex::Regex;
pub(crate) use ring::digest::{digest, SHA256};
pub(crate) use rsa::RSAPrivateKey;
pub(crate) use serde::de::DeserializeOwned;
pub(crate) use serde::{Deserialize, Serialize};
pub(crate) use serde_json::{from_str as from_json_str, to_string as to_json_string};
pub(crate) use strum::IntoEnumIterator;
pub(crate) use strum_macros::{AsRefStr, EnumIter, IntoStaticStr, ToString as StrumToString};
pub(crate) use validator::{Validate, ValidationErrors};

// The public middleware output type.
pub(crate) type MiddlewareFuture<T, E> =
    std::pin::Pin<Box<dyn std::future::Future<Output = Result<T, E>>>>;

pub(crate) type AuthServiceResult<T> = Result<T, AuthServiceError>;

pub(crate) const ROLES: [&str; 7] = [
    "developer",
    "task_admin",
    "processor_admin",
    "group_admin",
    "user_admin",
    "log_admin",
    "team_leader",
];
